from mayan.apps.appearance.classes import Icon

icon_current_user_events = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_current_user_events_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_event_types_subscriptions_list = Icon(
    driver_name='fontawesome', symbol='list-ol'
)
icon_events_for_object = Icon(driver_name='fontawesome', symbol='list-ol')
icon_events_for_object_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_events_list = Icon(driver_name='fontawesome', symbol='list-ol')
icon_events_list_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_notification_mark_read = Icon(
    driver_name='fontawesome', symbol='eye'
)
icon_notification_mark_read_all = Icon(
    driver_name='fontawesome', symbol='eye'
)
icon_object_event_types_user_subcriptions_list = Icon(
    driver_name='fontawesome', symbol='rss'
)
icon_user_notifications_list = Icon(
    driver_name='fontawesome', symbol='bell'
)
